
SUBROUTINE DECOMP(A,MAXA,NN,ISH,IOUT)
! . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
! .
! . P R O G R A M
! . TO CALCULATE (L)*(D)*(L)(T) FACTORIZATION OF
! . STIFFNESS MATRIX
! .
! . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
!
IMPLICIT NONE
REAL(8), INTENT(INOUT) :: A(1)
INTEGER, INTENT(IN) :: MAXA(1),NN,ISH,IOUT
!
INTEGER :: J,N,K,L,IC,KI,KN,KL,KU,KH,KK,KLT,ND
REAL(8) :: B,C
!
IF (NN.EQ.1) RETURN
!
DO N=1,NN
KN=MAXA(N)
KL=KN + 1
KU=MAXA(N+1) - 1
KH=KU - KL
IF (KH.GT.0) THEN
K=N - KH
IC=0
KLT=KU
DO J=1,KH
IC=IC + 1
KLT=KLT - 1
KI=MAXA(K)
ND=MAXA(K+1) - KI - 1
IF (ND.GT.0) THEN
KK=MIN0(IC,ND)
C=0.
DO L=1,KK
C=C + A(KI+L)*A(KLT+L)
END DO
A(KLT)=A(KLT) - C
END IF
K=K + 1
END DO
END IF
!
IF (KH.GE.0) THEN
K=N
B=0.
DO KK=KL,KU
K=K - 1
KI=MAXA(K)
C=A(KK)/A(KI)
IF (ABS(C).GE.1.E07) THEN
WRITE (IOUT,2010) N,C
STOP
END IF
B=B + C*A(KK)
A(KK)=C
END DO
A(KN)=A(KN) - B
END IF
!
IF (A(KN).LE.0) THEN
IF (ISH.EQ.0) THEN
WRITE (IOUT,2000) N,A(KN)
STOP
END IF
IF (A(KN).EQ.0.) A(KN)=-1.E-16
END IF
END DO
!
RETURN
!
2000 FORMAT (//' STOP - STIFFNESS MATRIX NOT POSITIVE DEFINITE',//, &
' NONPOSITIVE PIVOT FOR EQUATION ',I8,//, &
' PIVOT = ',E20.12)
2010 FORMAT (//' STOP - STURM SEQUENCE CHECK FAILED BECAUSE OF', &
' MULTIPLIER GROWTH FOR COLUMN NUMBER ',I8,//, &
' MULTIPLIER = ',E20.8)
END


SUBROUTINE REDBAK (A,V,MAXA,NN)
! . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
! .
! . P R O G R A M
! . TO REDUCE AND BACK-SUBSTITUTE ITERATION VECTORS
! .
! . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
!
IMPLICIT NONE
!
REAL(8), INTENT(INOUT) :: A(1),V(1)
INTEGER, INTENT(IN) ::NN,MAXA(1)
!
INTEGER :: L,K,N,KL,KU,KK
REAL(8) :: C
!
DO N=1,NN
KL=MAXA(N) + 1
KU=MAXA(N+1) - 1
IF (KU-KL.LT.0) CYCLE
K=N
C=0.
DO KK=KL,KU
K=K - 1
C=C + A(KK)*V(K)
END DO
V(N)=V(N) - C
END DO
!
DO N=1,NN
K=MAXA(N)
V(N)=V(N)/A(K)
END DO
!
IF (NN.EQ.1) RETURN
!
N=NN
DO L=2,NN
KL=MAXA(N) + 1
KU=MAXA(N+1) - 1
IF (KU-KL.LT.0) CYCLE
K=N
DO KK=KL,KU
K=K - 1
V(K)=V(K) - A(KK)*V(N)
END DO
N=N - 1
END DO
!
RETURN
END


SUBROUTINE MULT (TT,B,RR,MAXA,NN,NWM)
! . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
! .
! . P R O G R A M
! . TO EVALUATE PRODUCT OF B TIMES RR AND STORE RESULT IN TT
! .
! . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
!
IMPLICIT NONE
REAL(8), INTENT(INOUT) :: TT(1)
REAL(8), INTENT(IN) :: B(1),RR(1)
INTEGER, INTENT(IN) :: NN,NWM,MAXA(1)
!
INTEGER :: I,II,KL,KU,KK
REAL(8) :: AA,CC
!
IF (NWM.LE.NN) THEN
DO I=1,NN
TT(I)=B(I)*RR(I)
END DO
RETURN
END IF
!
DO I=1,NN
TT(I)=0.
END DO
!
DO I=1,NN
KL=MAXA(I)
KU=MAXA(I+1) - 1
II=I + 1
CC=RR(I)
DO KK=KL,KU
II=II - 1
TT(II)=TT(II) + B(KK)*CC
END DO
END DO
!
IF (NN.EQ.1) RETURN
!
DO I=2,NN
KL=MAXA(I) + 1
KU=MAXA(I+1) - 1
IF (KU-KL.LT.0) CYCLE
II=I
AA=0.
DO KK=KL,KU
II=II - 1
AA=AA + B(KK)*RR(II)
END DO
TT(I)=TT(I) + AA
END DO
!
RETURN
END


SUBROUTINE SCHECK(EIGV,RTOLV,BUP,BLO,BUPC,NEIV,NC,NEI,RTOL,SHIFT,IOUT)
! . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
! .
! . P R O G R A M
! . TO EVALUATE SHIFT FOR STURM SEQUENCE CHECK
! .
! . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
!
IMPLICIT NONE
INTEGER, INTENT(IN) :: NC,IOUT
REAL(8), INTENT(IN) :: EIGV(NC),RTOLV(NC),RTOL
REAL(8), INTENT(INOUT) :: BUP(NC),BUPC(NC),BLO(NC),SHIFT
INTEGER, INTENT(INOUT) :: NEI,NEIV(NC)
!
INTEGER :: I,L,LM,LL,NROOT
REAL(8) :: FTOL
!
FTOL=0.01
!
DO I=1,NC
BUP(I)=EIGV(I)*(1.+FTOL)
BLO(I)=EIGV(I)*(1.-FTOL)
END DO
!
NROOT=0
DO I=1,NC
IF (RTOLV(I).LT.RTOL) NROOT=NROOT + 1
END DO
!
IF (NROOT.LT.1) THEN
WRITE (IOUT,1010)
STOP
END IF
!
! FIND UPPER BOUNDS ON EIGENVALUE CLUSTERS
!
DO I=1,NROOT
NEIV(I)=1
END DO
!
IF (NROOT.EQ.1) THEN
BUPC(1)=BUP(1)
LM=1
L=1
I=2
ELSE
L=1
I=2
DO
DO
IF (BUP(I-1).LE.BLO(I)) EXIT
NEIV(L)=NEIV(L) + 1
I=I + 1
IF (I.GT.NROOT) EXIT
END DO
BUPC(L)=BUP(I-1)
IF (I.GT.NROOT) GO TO 290
L=L + 1
I=I + 1
IF (I.GT.NROOT) EXIT
END DO
BUPC(L)=BUP(I-1)
290 LM=L
IF (NROOT.EQ.NC) GO TO 300
END IF
!
DO
IF (BUP(I-1).LE.BLO(I)) EXIT
IF (RTOLV(I).GT.RTOL) EXIT
BUPC(L)=BUP(I)
NEIV(L)=NEIV(L) + 1
NROOT=NROOT + 1
IF (NROOT.EQ.NC) EXIT
I=I + 1
END DO
!
! FIND SHIFT
!
300 WRITE (IOUT,1020)
WRITE (IOUT,1005) (BUPC(I),I=1,LM)
WRITE (IOUT,1030)
WRITE (IOUT,1006) (NEIV(I),I=1,LM)
!
LL=LM - 1
IF (LM.NE.1) THEN
DO
DO I=1,LL
NEIV(L)=NEIV(L) + NEIV(I)
END DO
L=L - 1
LL=LL - 1
IF (L.EQ.1) EXIT
END DO
END IF
!
WRITE (IOUT,1040)
WRITE (IOUT,1006) (NEIV(I),I=1,LM)
!
L=0
DO I=1,LM
L=L + 1
IF (NEIV(I).GE.NROOT) EXIT
END DO
SHIFT=BUPC(L)
NEI=NEIV(L)
!
RETURN
!
1005 FORMAT (' ',6E22.14)
1006 FORMAT (' ',6I22)
1010 FORMAT (' *** ERROR *** SOLUTION STOP IN *SCHECK*',/, &
' NO EIGENVALUES FOUND',/)
1020 FORMAT (///,' UPPER BOUNDS ON EIGENVALUE CLUSTERS')
1030 FORMAT (//,' NO. OF EIGENVALUES IN EACH CLUSTER')
1040 FORMAT (' NO. OF EIGENVALUES LESS THAN UPPER BOUNDS')
END




